import typing
from collections.abc import Sequence
from pathlib import Path

from gwproactor import (
    App,
    AppSettings,
    LinkSettings,
    Proactor,
    ProactorName,
    WebEventListener,
)
from gwproactor.actors.web_event_listener import WebEventListenerSettings
from gwproactor.app import ActorConfig
from gwproactor.config import MQTTClient
from gwproactor.persister import TimedRollingFilePersister
from gwproto import HardwareLayout
from gwproto.named_types.web_server_gt import WebServerGt
from pydantic_settings import SettingsConfigDict

DEFAULT_UPLOADER_SHORT_NAME: str = "u"
DEFAULT_UPLOADER_PATHS_NAME = "uploader"
DEFAULT_INGESTER_SHORT_NAME: str = "i"


class UploaderSettings(AppSettings):
    long_name: str = ""
    short_name: str = DEFAULT_UPLOADER_SHORT_NAME
    ingester_long_name: str = ""
    ingester_short_name: str = DEFAULT_INGESTER_SHORT_NAME
    ingester: MQTTClient = MQTTClient()
    server: WebServerGt = WebServerGt()
    listener: WebEventListenerSettings = WebEventListenerSettings()

    model_config = SettingsConfigDict(
        env_prefix="UPLOADER_APP_",
    )


class UploaderApp(App):
    INGESTER_LINK: str = "ingester"

    @classmethod
    def app_settings_type(cls) -> type[UploaderSettings]:
        return UploaderSettings

    @property
    def settings(self) -> UploaderSettings:
        return typing.cast("UploaderSettings", self._settings)

    @classmethod
    def paths_name(cls) -> str:
        return DEFAULT_UPLOADER_PATHS_NAME

    def _load_hardware_layout(self, layout_path: str | Path) -> HardwareLayout:  # noqa: ARG002
        return HardwareLayout(
            layout={},
            cacs={},
            components={},
            nodes={},
            data_channels={},
            synth_channels={},
        )

    def _get_name(self, layout: HardwareLayout) -> ProactorName:  # noqa: ARG002
        return ProactorName(
            long_name=self.settings.long_name,
            short_name=self.settings.short_name,
        )

    def _get_link_settings(
        self,
        name: ProactorName,  # noqa: ARG002
        layout: HardwareLayout,  # noqa: ARG002
        brokers: dict[str, MQTTClient],  # noqa: ARG002
    ) -> dict[str, LinkSettings]:
        return {
            self.INGESTER_LINK: LinkSettings(
                broker_name=self.INGESTER_LINK,
                peer_long_name=self.settings.ingester_long_name,
                peer_short_name=self.settings.ingester_short_name,
                upstream=True,
            ),
        }

    def _instantiate_proactor(self) -> Proactor:
        proactor = super()._instantiate_proactor()
        proactor.add_web_server_config(
            name=self.settings.server.Name,
            host=self.settings.server.Host,
            port=self.settings.server.Port,
            enabled=self.settings.server.Enabled,
            server_kwargs=self.settings.server.Kwargs,
        )
        return proactor

    def _get_actor_nodes(self) -> Sequence[ActorConfig]:
        return [
            ActorConfig(
                node=self._layout.add_node(
                    node=WebEventListener.default_node(),
                ),
                constructor_args={"settings": self.settings.listener},
            )
        ]

    @classmethod
    def _make_persister(cls, settings: AppSettings) -> TimedRollingFilePersister:
        return TimedRollingFilePersister(settings.paths.event_dir)



import asyncio

import dotenv
import typer
from gwproactor.command_line_utils import run_async_main
from gwproactor.logging_setup import enable_aiohttp_logging
from gwproactor_test.certs import generate_dummy_certs

from gwupload import UploaderApp
from gwupload.stubs import stubs_cli

cli_app = typer.Typer(
    no_args_is_help=True,
    pretty_exceptions_enable=False,
    rich_markup_mode="rich",
    help="GridWorks Uploader",
)

cli_app.add_typer(stubs_cli.app, name="stubs", help="Use stub applications for testing")


@cli_app.command()
def run(
    env_file: str = ".env",
    *,
    dry_run: bool = False,
    verbose: bool = False,
    message_summary: bool = False,
    aiohttp_verbose: bool = False,
) -> None:
    """Run the uploader."""
    if aiohttp_verbose:
        enable_aiohttp_logging()
    asyncio.run(
        run_async_main(
            app_type=UploaderApp,
            env_file=env_file,
            dry_run=dry_run,
            verbose=verbose,
            message_summary=message_summary,
        )
    )


@cli_app.command()
def config(
    env_file: str = ".env",
) -> None:
    """Show uploader configuration"""
    UploaderApp.print_settings(env_file=env_file)


@cli_app.command()
def gen_test_certs(*, dry_run: bool = False, env_file: str = ".env") -> None:
    """Generate test certs for the uploader."""
    generate_dummy_certs(
        UploaderApp(env_file=dotenv.find_dotenv(env_file, usecwd=True)).settings,
        dry_run=dry_run,
    )


@cli_app.callback()
def _main() -> None: ...


if __name__ == "__main__":
    cli_app()



"""Scada implementation"""

import logging
import typing
from logging import Logger, LoggerAdapter
from pathlib import Path
from typing import Any, Optional

from gwproactor import App, AppSettings, ServicesInterface
from gwproactor.actors.actor import PrimeActor
from gwproactor.config import MQTTClient
from gwproactor.config.links import LinkSettings
from gwproactor.config.proactor_config import ProactorName
from gwproactor.message import MQTTReceiptPayload
from gwproactor.persister import PersisterInterface, SimpleDirectoryWriter
from gwproto import HardwareLayout, Message
from gwproto.messages import EventBase
from pydantic_settings import SettingsConfigDict

from gwupload.app import DEFAULT_INGESTER_SHORT_NAME, DEFAULT_UPLOADER_SHORT_NAME
from gwupload.stubs.names import STUB_INGESTER_LONG_NAME


class StubIngesterSettings(AppSettings):
    long_name: str = STUB_INGESTER_LONG_NAME
    short_name: str = DEFAULT_INGESTER_SHORT_NAME
    uploader_long_name: str = ""
    uploader_short_name: str = DEFAULT_UPLOADER_SHORT_NAME
    uploader: MQTTClient = MQTTClient()
    event_logger_level: int = logging.WARNING
    event_logger_content_len: int = 80

    model_config = SettingsConfigDict(
        env_prefix="STUB_INGESTER_APP_",
    )


class StubIngester(PrimeActor):
    EVENT_LOGGER_NAME: str = "events"
    event_logger: Logger | LoggerAdapter[Logger]

    def __init__(self, name: str, services: ServicesInterface) -> None:
        super().__init__(name, services)
        self.event_logger = self.services.logger.add_category_logger(
            category=self.EVENT_LOGGER_NAME,
            level=self.settings.event_logger_level,
        )

    @property
    def settings(self) -> StubIngesterSettings:
        return typing.cast("StubIngesterSettings", self.services.settings)

    def event_str(self, event: EventBase) -> str:
        content_str = event.model_dump_json(
            exclude={"MessageId", "TimeCreatedMs", "Src", "TypeName", "Version"}
        )
        if len(content_str) > self.settings.event_logger_content_len - 3:
            content_str = (
                content_str[: self.settings.event_logger_content_len - 3] + "..."
            )
        type_str = f"<{event.TypeName}>"
        return f"Received {type_str:50s}  {content_str}"

    def log_event(self, event: EventBase) -> None:
        self.event_logger.info(self.event_str(event))

    def process_mqtt_message(
        self, _: Message[MQTTReceiptPayload], decoded: Message[Any]
    ) -> None:
        if isinstance(decoded.Payload, EventBase):
            self.log_event(decoded.Payload)


class StubIngesterApp(App):
    UPLOADER_LINK: str = "uploader"

    @classmethod
    def app_settings_type(cls) -> type[StubIngesterSettings]:
        return StubIngesterSettings

    @property
    def settings(self) -> StubIngesterSettings:
        return typing.cast("StubIngesterSettings", self._settings)

    @classmethod
    def prime_actor_type(cls) -> type[StubIngester]:
        return StubIngester

    @classmethod
    def paths_name(cls) -> Optional[str]:
        return STUB_INGESTER_LONG_NAME

    def _get_name(self, layout: HardwareLayout) -> ProactorName:  # noqa: ARG002
        return ProactorName(
            long_name=self.settings.long_name,
            short_name=self.settings.short_name,
        )

    def _load_hardware_layout(self, layout_path: str | Path) -> HardwareLayout:  # noqa: ARG002
        return HardwareLayout(
            layout={},
            cacs={},
            components={},
            nodes={},
            data_channels={},
            synth_channels={},
        )

    def _get_link_settings(
        self,
        name: ProactorName,  # noqa: ARG002
        layout: HardwareLayout,  # noqa: ARG002
        brokers: dict[str, MQTTClient],  # noqa: ARG002
    ) -> dict[str, LinkSettings]:
        return {
            self.UPLOADER_LINK: LinkSettings(
                broker_name=self.UPLOADER_LINK,
                peer_long_name=self.settings.uploader_long_name,
                peer_short_name=self.settings.uploader_short_name,
                downstream=True,
            )
        }

    @classmethod
    def _make_persister(cls, settings: AppSettings) -> PersisterInterface:
        return SimpleDirectoryWriter(settings.paths.event_dir)

    @classmethod
    def get_settings(cls, *args: Any, **kwargs: Any) -> StubIngesterSettings:
        return typing.cast(
            "StubIngesterSettings", super().get_settings(*args, **kwargs)
        )



import random
import time
from typing import Literal

import httpx
from gwproto.messages import EventBase


class SomeData(EventBase):
    TimestampUTC: float
    Reading: float
    TypeName: Literal["gridworks.event.some.data"] = "gridworks.event.some.data"


if __name__ == "__main__":
    httpx.post(
        "http://127.0.0.1:8080/events",
        json=SomeData(
            TimestampUTC=round(time.time(), 3),
            Reading=round(random.random(), 3),
        ).model_dump(),
    )



import asyncio
import logging

import dotenv
import typer
from gwproactor.command_line_utils import run_async_main
from gwproactor_test.certs import generate_dummy_certs

from gwupload.stubs.ingester.ingester import StubIngesterApp

app = typer.Typer(
    no_args_is_help=True,
    pretty_exceptions_enable=False,
    rich_markup_mode="rich",
    help="GridWorks Dummy Ingester",
)


@app.command()
def run(
    *,
    env_file: str = ".env",
    dry_run: bool = False,
    verbose: bool = False,
    message_summary: bool = False,
    log_events: bool = False,
) -> None:
    """Run the stub ingester"""

    env_file = dotenv.find_dotenv(filename=env_file, usecwd=True)
    app_settings = StubIngesterApp.get_settings(
        env_file=env_file,
    )
    if log_events:
        app_settings.event_logger_level = logging.INFO
    asyncio.run(
        run_async_main(
            app_settings=app_settings,
            app_type=StubIngesterApp,
            env_file=env_file,
            dry_run=dry_run,
            verbose=verbose,
            message_summary=message_summary,
        )
    )


@app.command()
def gen_test_certs(*, dry_run: bool = False, env_file: str = ".env") -> None:
    """Generate test certs for the stub ingester."""
    generate_dummy_certs(
        StubIngesterApp(env_file=dotenv.find_dotenv(env_file, usecwd=True)).settings,
        dry_run=dry_run,
    )


@app.command()
def config(
    env_file: str = ".env",
) -> None:
    """Show stub ingester configuration"""
    StubIngesterApp.print_settings(env_file=env_file)


@app.callback()
def _main() -> None: ...


if __name__ == "__main__":
    app()
